"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteBatchSchedule = exports.SaveBatchSchedule = exports.GetBatchSchedule = exports.BatchScheduleRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function BatchScheduleRedis(app) {
    redisClient = app.get('redisClient');
}
exports.BatchScheduleRedis = BatchScheduleRedis;
function GetBatchSchedule(key) {
    var storage = 'BatchScheduleStorage';
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var batchSchedule = null;
        try {
            batchSchedule = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(batchSchedule);
    })
        .catch(function (err) {
        logger.error('[BatchSheduleRedis::GetBatchSchedule] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetBatchSchedule = GetBatchSchedule;
function SaveBatchSchedule(key, data) {
    var storage = 'BatchScheduleStorage';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(data))
        .then(function (_) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchSheduleRedis::SaveBatchSchedule] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveBatchSchedule = SaveBatchSchedule;
function DeleteBatchSchedule(key) {
    var storage = 'BatchScheduleStorage';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchSheduleRedis::BatchScheduleStorage] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteBatchSchedule = DeleteBatchSchedule;
