"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllReadyTipificationInteractionStorages = exports.GetReadyTipificationInteractions = exports.SaveReadyTipificationInteraction = exports.DeleteTipificationInteraction = exports.SaveTipificationInteraction = exports.GetTipificationInteraction = exports.TipificationRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function TipificationRedis(app) {
    redisClient = app.get('redisClient');
}
exports.TipificationRedis = TipificationRedis;
function GetTipificationInteraction(key) {
    var storage = 'Tipification';
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch(function (err) {
        logger.error('[TipificationRedis::GetTipificationInteraction] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetTipificationInteraction = GetTipificationInteraction;
function SaveTipificationInteraction(key, interaction) {
    var storage = 'Tipification';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[TipificationRedis::SaveTipificationInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveTipificationInteraction = SaveTipificationInteraction;
function DeleteTipificationInteraction(key) {
    var storage = 'Tipification';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[TipificationRedis::DeleteTipificationInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteTipificationInteraction = DeleteTipificationInteraction;
function SaveReadyTipificationInteraction(key, interaction, executionCode) {
    var storage = 'ReadyTipificationInteraction';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(function (result) {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[TipificationRedis::SaveReadyTipificationInteraction] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyTipificationInteraction = SaveReadyTipificationInteraction;
function GetReadyTipificationInteractions(executionCode) {
    var hash = 'ReadyTipificationInteraction:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then(function (result) {
        var interactions = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[TipificationRedis::GetReadyTipificationInteraction] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch(function (err) {
        logger.error('[TipificationRedis::GetReadyTipificationInteraction] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyTipificationInteractions = GetReadyTipificationInteractions;
function GetAllReadyTipificationInteractionStorages(executionCode) {
    var pattern = 'inconcertCache:ReadyTipificationInteraction:*';
    return redisClient.keysAsync([pattern])
        .then(function (result) {
        if ((result === null || result === void 0 ? void 0 : result.length) > 1)
            logger.info("[TipificationRedis::ReadyTipificationInteraction] Getting " + result.length + " storages for execution " + executionCode);
        var executionCodes = [];
        result.forEach(function (r) {
            executionCodes.push(r.replace('inconcertCache:ReadyTipificationInteraction:', ''));
        });
        executionCodes = executionCodes.sort();
        return executionCodes;
    })
        .catch(function (err) {
        logger.error('[TipificationRedis::ReadyTipificationInteraction] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllReadyTipificationInteractionStorages = GetAllReadyTipificationInteractionStorages;
