"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveReports = exports.SaveInteractionTypes = exports.SaveAzureSubscriptionKeys = exports.SaveAzureSubscription = exports.SaveSystemDictionaries = exports.SaveSystemApp = exports.SaveTenantVCC = exports.FilterVCCLicense = exports.SaveVCCLicense = exports.SaveVCC = exports.GetVCC = exports.UpdateLicense = exports.VCCService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_config_1 = require("../config/mssql.config");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var integration_config_1 = require("../config/integration.config");
var user_service_1 = require("../services/user.service");
var promises_shared_1 = require("../shared/promises.shared");
var billing_implementation_1 = require("../implementation/billing.implementation");
var billing_account_implementation_1 = require("../implementation/billing-account.implementation");
var redis_service_1 = require("../db/redis.service");
var cassandra_service_1 = require("../db/cassandra.service");
var cassandra_init_1 = require("../db/cassandra.init");
var nlu_service_1 = require("./nlu.service");
var intent_service_1 = require("./intent.service");
var cognitive_engine_migration_model_1 = require("../models/cognitive-engine-migration.model");
var logger = log4js.getLogger('ServerScripts');
function VCCService(app) {
    app.get('/api/vcc/get_vccs/:search_criteria/:search_expression?', function (req, res, next) {
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetVCCs(vcc, search_criteria, search_expression);
        }).then(function (vccs) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: vccs });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::/api/vcc/get_vccs/] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/vcc/get_vcc/:vcc_id', function (req, res, next) {
        var vcc = res.locals.vcc;
        var vccId = req.params.vcc_id;
        var accountDetail;
        var products;
        var accountId;
        var subAccountId;
        var assignedAccount = 0;
        var availableAccount = 0;
        var transcriptionRate = 0;
        var semanticRate = 0;
        var cognitiveRate = 0;
        var azureSubcription = {};
        var azureSubscriptionKeys = [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetVCCBalance(vccId);
        }).then(function (result) {
            accountDetail = result;
            accountId = result[0].accountId;
            subAccountId = result[0].subAccountId;
            result.forEach(function (x) {
                assignedAccount += x.countableBalance;
            });
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetVCCStatusDetail(vccId);
        }).then(function (result) {
            products = result.products;
            var billing = new billing_account_implementation_1.BillingAccountImplementation();
            return billing.GetAccountBalance(accountDetail[0].accountId);
        }).then(function (result) {
            if (!result.status)
                throw 'ERROR_TO_GET_ACCOUNT_DATA';
            if (subAccountId) {
                var getBalance = result.data.subAccountsBalance.find(function (x) { return x.subaccountId === subAccountId; });
                var getRate = result.data.subAccountsRate.find(function (x) { return x.subaccountId === subAccountId; });
                availableAccount = getBalance.availableBalance;
                transcriptionRate = getRate.transcriptionRatePerHour;
                semanticRate = getRate.semanticAnalysisRatePerTransaction;
                cognitiveRate = getRate.cognitiveAnalysisRatePerTransaction;
            }
            else {
                var getBalance = result.data.accountBalance;
                var getRate = result.data.accountRate;
                availableAccount = getBalance.availableBalance;
                transcriptionRate = getRate.transcriptionRatePerHour;
                semanticRate = getRate.semanticAnalysisRatePerTransaction;
                cognitiveRate = getRate.cognitiveAnalysisRatePerTransaction;
            }
            if (!vccId)
                throw 'VCC_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetAzureSubscription(vccId);
        })
            .then(function (result) {
            if (result !== null)
                azureSubcription = result;
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(vccId);
        })
            .then(function (result) {
            if (result !== null)
                azureSubscriptionKeys = result;
            return mssql_service_1.MSSql.GetVCC(vccId);
        }).then(function (resultVCC) {
            if (res.headersSent)
                return;
            var vcc = resultVCC;
            vcc.productVCC = {};
            vcc.productVCC.accountId = accountId;
            vcc.productVCC.subAccountId = subAccountId;
            vcc.productVCC.asignableBalance = assignedAccount;
            vcc.productVCC.availableBalance = availableAccount;
            vcc.productVCC.transcriptionRatePerHour = transcriptionRate;
            vcc.productVCC.semanticAnalysisRatePerTransaction = semanticRate;
            vcc.productVCC.cognitiveAnalysisRatePerTransaction = cognitiveRate;
            vcc.productVCC.products = [];
            vcc.azureSubscription = azureSubcription;
            vcc.azureSubscriptionKeyMain = azureSubscriptionKeys.find(function (ask) { return ask.subscriptionName === 'DEMO'; });
            var azureSubscriptionKeyProd = azureSubscriptionKeys.filter(function (ask) { return ask.isDemo === false; });
            if (azureSubscriptionKeyProd.length >= 1)
                vcc.azureSubscriptionKey1 = azureSubscriptionKeyProd[0];
            if (azureSubscriptionKeyProd.length >= 2)
                vcc.azureSubscriptionKey2 = azureSubscriptionKeyProd[1];
            products.forEach(function (x) {
                var product = {};
                product.product = x.product;
                product.enabled = x.enabled;
                vcc.productVCC.products.push(product);
            });
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultVCC });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::get_vcc] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/get_integrated_vccs', function (req, res, next) {
        var vcc = res.locals.vcc;
        var cassandraConn = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (integration_config_1.IP_I6 !== '')
                return cassandra_init_1.CassandralInit(app);
            else
                return true;
        })
            .then(function (result) {
            var type = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetVCCs(vcc, '', ''));
            if (integration_config_1.IP_BD !== '')
                promiseArray.push(mssql_service_1.MSSql.GetIntegratedVCCs(vcc, integration_config_1.IP_BD, type));
            else
                promiseArray.push(Promise.resolve([]));
            if (integration_config_1.IP_I6 !== '') {
                cassandraConn = app.get('cassandraConn');
                promiseArray.push(cassandra_service_1.CassandraSql.GetCassandraVCCs());
            }
            else
                promiseArray.push(Promise.resolve([]));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (!result)
                'NOT_FOUND';
            var integratedVCCs = [];
            var speechVCCs = result[0] ? result[0] : [];
            var sqlServerVCCs = result[1] ? result[1] : [];
            var cassandraVCCs = result[2] ? result[2] : [];
            sqlServerVCCs.map(function (vcc) {
                if (vcc && vcc.vcc && vcc.vcc !== 'system') {
                    var integratedVCC = {};
                    integratedVCC.vcc = vcc.vcc;
                    integratedVCC.type = vcc.hasOmnichannel === 1 ? 'OCC' : 'ALLEGRO';
                    integratedVCCs.push(integratedVCC);
                }
            });
            cassandraVCCs.map(function (vcc) {
                if (vcc && vcc.vcc && vcc.vcc !== 'system') {
                    var integratedVCC = {};
                    integratedVCC.vcc = vcc.vcc;
                    integratedVCC.type = vcc.allegroIntegration.includes('true') ? 'OCC' : 'I6';
                    integratedVCCs.push(integratedVCC);
                }
            });
            if (integration_config_1.IP_I6 !== '') {
                cassandraConn.close();
            }
            integratedVCCs = integratedVCCs.reduce(function (acc, next, index, src) {
                var duplicatedVcc = acc.find(function (v) { return v.vcc === next.vcc; });
                if (duplicatedVcc) {
                    if (duplicatedVcc.type !== next.type) {
                        duplicatedVcc.info = 'SERVER_ERROR_INCONSISTENT_VCC_INTEGRATION_DATA';
                        duplicatedVcc.enabled = false;
                    }
                }
                else {
                    next.enabled = true;
                    if (speechVCCs.find(function (v) { return v.id === next.vcc; })) {
                        next.isUsed = true;
                        next.info = 'SERVER_ERROR_VCC_IS_ALREADY_REGISTERED';
                    }
                    else {
                        next.isUsed = false;
                        acc.push(next);
                    }
                }
                return acc;
            }, []);
            return integratedVCCs;
        })
            .then(function (vccs) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: vccs });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::get_integrated_vccs] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/create_vcc', function (req, res, next) {
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newVCC = (req.body.vcc || {});
        var newUser = (req.body.user || {});
        newVCC.id = (newVCC.id || '').toLowerCase();
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.isSystemUser = false;
        newUser.role = 'administrator';
        newVCC.createdDate = nowUtc;
        newVCC.createdByUserId = userId;
        newVCC.lastModifiedDate = nowUtc;
        newVCC.lastModifiedByUserId = userId;
        newVCC.type = newVCC.type;
        newVCC.productVCC.accountId = newVCC.productVCC.accountId;
        newVCC.productVCC.subAccountId = newVCC.productVCC.subAccountId;
        newVCC.productVCC.products = newVCC.productVCC.products;
        newVCC.productVCC.asignableBalance = newVCC.productVCC.asignableBalance;
        newUser.firstname = 'user' + newVCC.id;
        newUser.createdDate = nowUtc;
        newUser.createdByUserId = userId;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        newUser.language = newVCC.language;
        vcc = newVCC.id;
        var errors = [];
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return newVCC.id.trim() === config_1.IDATHA_TEST_REALM;
        })
            .then(function (result) {
            if (!result) {
                return mssql_service_1.MSSql.GetVCC(newVCC.id);
            }
            return result;
        })
            .then(function (resultVCC) {
            if (resultVCC) {
                throw 'VCC_SERVER_ERROR_EXISTS';
            }
            var products = [];
            if (newVCC.productVCC.products && newVCC.productVCC.products.length) {
                products = newVCC.productVCC.products.map(function (x) {
                    return x.product;
                });
            }
            return SaveVCCLicense(products, 1, vcc, userId, nowUtc);
        })
            .then(function (result) {
            if (result) {
                return SaveAzureSubscription(newVCC.azureSubscription, vcc);
            }
            throw 'FAILED_TO_SAVE_LICENSE';
        })
            .then(function (result) {
            if (!result) {
                errors.push('SERVER_ERROR_AZURE_SUBSCRIPTION_SAVING_FAILED');
                return false;
            }
            var azureSubscriptionKeyArray = [];
            if (newVCC.azureSubscriptionKeyMain.subscriptionKey !== null)
                azureSubscriptionKeyArray.push(newVCC.azureSubscriptionKeyMain);
            if (newVCC.azureSubscriptionKey1.subscriptionKey !== null)
                azureSubscriptionKeyArray.push(newVCC.azureSubscriptionKey1);
            if (newVCC.azureSubscriptionKey1.subscriptionKey !== null)
                azureSubscriptionKeyArray.push(newVCC.azureSubscriptionKey2);
            return SaveAzureSubscriptionKeys(azureSubscriptionKeyArray, vcc);
        })
            .then(function (result) {
            if (!result) {
                errors.push('SERVER_ERROR_AZURE_SUBSCRIPTION_KEYS_SAVING_FAILED');
                return false;
            }
            return SaveVCC(vcc, newVCC);
        })
            .then(function (result) {
            if (!result) {
                errors.push('SERVER_ERROR_VCC_SAVING_FAILED');
                return false;
            }
            return user_service_1.SaveUser(vcc, newUser);
        })
            .then(function (result) {
            var promises = [];
            if (!result || errors.length > 0) {
                promises.push(Promise.resolve(false));
                promises.push(mssql_service_1.MSSql.RollbackVcc(vcc));
                errors.push('SERVER_ERROR_USER_SAVING_FAILED');
            }
            else {
                promises.push(Promise.resolve(true));
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            var result = results[0];
            if (!result) {
                throw errors[0];
            }
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.CreateVCC(vcc, newVCC.productVCC);
        })
            .then(function (result) {
            logger.info('[VCCService::create_vcc] Getting result:', result, 'after billing.CreateVCC');
            var promises = [];
            if (result) {
                if (result.status && result.status === true) {
                    promises.push(Promise.resolve(true));
                }
                else {
                    promises.push(Promise.resolve(false));
                    promises.push(Promise.resolve(result.error ? result.error : result.description));
                    promises.push(mssql_service_1.MSSql.RollbackVcc(vcc));
                }
            }
            else {
                promises.push(Promise.resolve(false));
                promises.push(Promise.resolve('FAILED_TO_SAVE_VCC_IN_EXTERNAL_SERVER'));
                promises.push(mssql_service_1.MSSql.RollbackVcc(vcc));
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            if (res.headersSent)
                return;
            var resultSave = results[0];
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : results[1]), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::create_vcc] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/save_vcc', function (req, res, next) {
        logger.info('[VCCService::save_vcc] Getting req.method:', req.method, 'req.path:', req.path);
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var updatedVCC = (req.body || {});
        updatedVCC.id = (updatedVCC.id || '').toLowerCase();
        updatedVCC.createdDate = null;
        updatedVCC.createdByUserId = null;
        updatedVCC.lastModifiedDate = nowUtc;
        updatedVCC.lastModifiedByUserId = userId;
        vcc = updatedVCC.id;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return SaveVCC(vcc, updatedVCC);
        })
            .then(function (result) {
            if (!result)
                throw 'VCC_SERVER_ERROR_ON_SAVE';
            return SaveAzureSubscription(updatedVCC.azureSubscription, vcc);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AZURE_SUBSCRIPTION_SAVING_FAILED';
            var azureSubscriptionKeyArray = [];
            if (updatedVCC.azureSubscriptionKeyMain.subscriptionKey !== null)
                azureSubscriptionKeyArray.push(updatedVCC.azureSubscriptionKeyMain);
            if (updatedVCC.azureSubscriptionKey1.subscriptionKey !== null)
                azureSubscriptionKeyArray.push(updatedVCC.azureSubscriptionKey1);
            if (updatedVCC.azureSubscriptionKey2.subscriptionKey !== null)
                azureSubscriptionKeyArray.push(updatedVCC.azureSubscriptionKey2);
            return SaveAzureSubscriptionKeys(azureSubscriptionKeyArray, vcc);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AZURE_SUBSCRIPTION_KEYS_SAVING_FAILED';
            return UpdateLicense(updatedVCC.productVCC.products, vcc, updatedVCC.createdDate, updatedVCC.createdByUserId, nowUtc, userId);
        })
            .then(function (resultSaveVCC) {
            if (!resultSaveVCC)
                throw 'ERROR_TO_UPDATE_LICENSE';
            var promiseArray = [];
            promiseArray.push(redis_service_1.Redis.DeleteVCC(vcc));
            promiseArray.push(redis_service_1.Redis.DeleteRequestData(vcc));
            promiseArray.push(redis_service_1.Redis.DeleteSubscriptionKey(vcc));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (!result)
                logger.info('[VCCService::save_vcc] Cleaning redis keys VCC:RequestData, azure failed');
            var billing = new billing_implementation_1.BillingImplementation();
            var products = [];
            for (var _i = 0, _a = updatedVCC.productVCC.products; _i < _a.length; _i++) {
                var p = _a[_i];
                if (p.assign) {
                    products.push(p);
                }
            }
            updatedVCC.productVCC.products = products;
            return billing.UpdateVCC(vcc, updatedVCC.productVCC);
        })
            .then(function (result) {
            if (updatedVCC.totalNlu && updatedVCC.totalNlu > 0 &&
                updatedVCC.changedCognitiveEngine && !updatedVCC.inUseModels) {
                redis_service_1.Redis.AddTaskCognitiveEngineMigration(updatedVCC.id, new cognitive_engine_migration_model_1.CognitiveEngineMigrationConfig(updatedVCC.oldCognitiveEngine, updatedVCC.cognitiveEngine, updatedVCC.azureSubscriptionKeyMain.subscriptionRegion, updatedVCC.azureSubscriptionKeyMain.subscriptionKey, updatedVCC.totalNlu, updatedVCC.totalIntent, updatedVCC.totalUtterance, updatedVCC.totalEntity));
            }
            return result;
        })
            .then(function (resultSaveVCC) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveVCC, description: (resultSaveVCC ? 'SERVER_SUCCESS_MESSAGE' : 'VCC_SERVER_ERROR_ON_SAVE'), data: resultSaveVCC });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::save_vcc] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/vcc/get_vcc_session', function (req, res, next) {
        var vcc = res.locals.vcc;
        var vcc_license;
        var vcc_redis;
        promises_shared_1.DummyPromise()
            .then(function (resultVCC) {
            return mssql_service_1.MSSql.GetVCCLicense(vcc);
        })
            .then(function (result) {
            vcc_license = (result || []);
            return redis_service_1.Redis.GetVCC(vcc, vcc);
        })
            .then(function (resultVCC) {
            vcc_redis = (resultVCC || {});
            var verify = FilterVCCLicense(vcc_license, vcc_redis.products);
            if (verify && verify.length === 1) {
                vcc_redis.products = vcc_redis.products.filter(function (x) { return x.product == verify[0]; });
            }
            if (verify && verify.length === 0) {
                vcc_redis.products = [];
            }
            return Promise.resolve(vcc_redis);
        })
            .then(function (resultVCC) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultVCC });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::/api/vcc/get_vcc_session] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/vcc/get_vcc_status', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetVCCStatus(vcc);
        }).then(function (result) {
            if (!result)
                throw 'ERROR_GET_VCC_STATUS_OF_EXTERNAL_SERVER';
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::/api/vcc/get_vcc_status] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/get_vcc_rates', function (req, res, next) {
        logger.info('[VCCService::/api/vcc/get_vcc_rates] Getting req.method:', req.method, 'req.path:', req.path);
        var vcc = res.locals.vcc;
        var processId = req.body.processId ? req.body.processId : null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetVCCRates(vcc, processId);
        }).then(function (result) {
            if (result === null)
                throw 'ERROR_NOT_RESULT_FROM_EXTERNAL_SERVER';
            logger.info('[VCCService::get_vcc_rates] Result from external server: ' + result, 'vcc:', vcc, 'after billing.GetVCCRates');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::/api/vcc/get_vcc_rates] Getting error:', err, 'vcc:', vcc, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/get_vcc_balance', function (req, res, next) {
        logger.info('[VCCService::/api/vcc/get_vcc_balance] Getting req.method:', req.method, 'req.path:', req.path);
        var vcc = res.locals.vcc;
        var vccBody = req.body.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            if (vcc === 'system') {
                return billing.GetVCCBalance(vccBody);
            }
            else {
                return billing.GetVCCBalance(vcc);
            }
        }).then(function (result) {
            if (result === null)
                throw 'ERROR_NOT_RESULT_FROM_EXTERNAL_SERVER';
            logger.info('[VCCService::get_vcc_balance] Result from external server: ' + result, 'vcc:', vcc, 'after billing.GetVCCBalance');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::get_vcc_balance] Getting error:', err, 'vcc:', vcc, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/save_vcc_balance', function (req, res, next) {
        logger.info('[VCCService::/api/vcc/save_vcc_balance] Getting req.method:', req.method, 'req.path:', req.path);
        var vcc = res.locals.vcc;
        var account = req.body.account;
        var subAccount = req.body.subAccount;
        var data = req.body.data;
        if (vcc === 'system') {
            vcc = req.body.vcc;
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!data && !account && !subAccount && !vcc)
                throw 'MISSING_PARAMETERS';
            var transcriptionNew = data.transcriptionNew;
            var transcriptionOld = data.transcriptionOld;
            var transcriptionDetail = (transcriptionNew == transcriptionOld) ? 'none' : (transcriptionNew > transcriptionOld) ? 'increase' : 'decrease';
            var transcriptionBalance = (transcriptionDetail == 'none') ? 0 : (transcriptionDetail == 'increase') ? transcriptionNew - transcriptionOld : transcriptionOld - transcriptionNew;
            var semanticNew = data.semanticNew;
            var semanticOld = data.semanticOld;
            var semanticDetail = (semanticNew == semanticOld) ? 'none' : (semanticNew > semanticOld) ? 'increase' : 'decrease';
            var semanticBalance = (semanticDetail == 'none') ? 0 : (semanticDetail == 'increase') ? semanticNew - semanticOld : semanticOld - semanticNew;
            var cognitiveNew = data.cognitiveNew;
            var cognitiveOld = data.cognitiveOld;
            var cognitiveDetail = (cognitiveNew == cognitiveOld) ? 'none' : (cognitiveNew > cognitiveOld) ? 'increase' : 'decrease';
            var cognitiveBalance = (cognitiveDetail == 'none') ? 0 : (cognitiveDetail == 'increase') ? cognitiveNew - cognitiveOld : cognitiveOld - cognitiveNew;
            var freeNew = data.freeNew;
            var freeOld = data.freeOld;
            var freeDetail = (freeNew == freeOld) ? 'none' : (freeNew > freeOld) ? 'increase' : 'decrease';
            var freeBalance = (freeDetail == 'none') ? 0 : (freeDetail == 'increase') ? freeNew - freeOld : freeOld - freeNew;
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.SaveVCCBalance(vcc, account, subAccount, transcriptionBalance, transcriptionNew, transcriptionDetail, cognitiveBalance, cognitiveNew, cognitiveDetail, semanticBalance, semanticNew, semanticDetail, freeBalance, freeNew, freeDetail);
        }).then(function (result) {
            logger.info('[VCCService::/api/vcc/save_vcc_balance] Result from external server: ' + result, 'vcc:', vcc, 'after billing.GetVCCBalance');
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result) ? 'SERVER_SUCCESS_MESSAGE' : 'ERROR', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[VCCService::/api/vcc/save_vcc_balance] Getting error:', err, 'vcc:', vcc, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.VCCService = VCCService;
function UpdateLicense(products, vcc, createdDate, createdByUserId, date, user) {
    var promiseArray = [];
    products.forEach(function (x) {
        promiseArray.push(mssql_service_1.MSSql.SaveVCCLicense(x.product, x.assign, vcc, createdDate, createdByUserId, date, user));
    });
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.UpdateLicense = UpdateLicense;
function GetVCC(vcc, vccId) {
    vccId = (vccId || '').toLowerCase();
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetVCC(vccId);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetVCC: ' + errMsg);
    });
}
exports.GetVCC = GetVCC;
function SaveVCC(vcc, objVCC) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveVCC(vcc, objVCC.id, objVCC.description, objVCC.type, objVCC.transcriptionDefaultLanguage, objVCC.cognitiveEngine, objVCC.country, objVCC.language, objVCC.timeZone, objVCC.createdDate, objVCC.createdByUserId, objVCC.lastModifiedDate, objVCC.lastModifiedByUserId, objVCC.deleteDataDays, objVCC.changedCognitiveEngine));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveVCC = SaveVCC;
function SaveVCCLicense(licenses, status, vcc, user, nowUtc) {
    var response = [];
    licenses.forEach(function (x) {
        response.push(mssql_service_1.MSSql.SaveVCCLicense(x, status, vcc, nowUtc, user, nowUtc, user));
    });
    if (!response.includes(false)) {
        return Promise.resolve(true);
    }
    else {
        return Promise.resolve(false);
    }
}
exports.SaveVCCLicense = SaveVCCLicense;
function FilterVCCLicense(vcc_bd, vcc_redis) {
    var results = [];
    if (vcc_redis && vcc_bd) {
        for (var i = 0; i < vcc_bd.length; i++) {
            for (var o = 0; o < vcc_redis.length; o++) {
                if (vcc_redis[o].product == vcc_bd[i].product && vcc_redis[o].availableLicense == vcc_bd[i].status) {
                    results.push(vcc_bd[i].product);
                }
            }
        }
    }
    return results;
}
exports.FilterVCCLicense = FilterVCCLicense;
function SaveTenantVCC(vcc, userId) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.GetTenantVCCByVCC('system'));
    return Promise.all(promiseArray)
        .then(function (result) {
        if (!result[0])
            throw 'SERVER_ERROR_TENANT_NOT_FOUND';
        var tenantVCC = {};
        tenantVCC = result[0];
        tenantVCC.createdUserId = userId;
        return mssql_service_1.MSSql.SaveTenantVCC(vcc, tenantVCC.tenantId, tenantVCC.clientId, tenantVCC.clientSecret, tenantVCC.resource, userId, null);
    })
        .then(function (result) {
        if (result) {
            return true;
        }
        else {
            return false;
        }
    });
}
exports.SaveTenantVCC = SaveTenantVCC;
function SaveSystemApp(vcc, userId) {
    var systemApps = [];
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetNLUs('system', null, null, null, true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_SYSTEM_APP_NOT_FOUND';
        systemApps = result ? result : [];
        var promiseArray = [];
        systemApps.map(function (sa) {
            sa.createdByUserId = userId;
            sa.appId = vcc + '::' + sa.name;
            promiseArray.push(nlu_service_1.SaveNLU(vcc, sa));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result.includes(false))
            throw 'SERVER_ERROR_SAVE_SYSTEM_APP_FAILED';
        var promiseArray = [];
        systemApps.filter(function (sa) { return sa.name.includes('diarization-'); }).map(function (sa) {
            var intent = {};
            intent.name = 'Agent';
            intent.intentId = 'Agent';
            intent.appId = sa.appId;
            intent.createdDate = moment().utc().toDate();
            intent.createdByUserId = userId,
                promiseArray.push(intent_service_1.SaveIntent(vcc, intent));
        });
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[VCCService::SaveSystemApp] Error: ' + error);
        return Promise.resolve(false);
    });
}
exports.SaveSystemApp = SaveSystemApp;
function SaveSystemDictionaries(vcc, userId) {
    var systemDiccionaries = [];
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetDictionaries('system', null, null);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_SYSTEM_DICTIONARIES_NOT_FOUND';
        systemDiccionaries = result ? result : [];
        var promiseArray = [];
        systemDiccionaries.map(function (d) {
            promiseArray.push(mssql_service_1.MSSql.SaveDictionary(vcc, d.name, d.description, d.language, true));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result.includes(false))
            throw 'SERVER_ERROR_SAVE_SYSTEM_DICTIONARIES_FAILED';
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[VCCService::SaveSystemDictionaries] Error: ' + error);
        return Promise.resolve(false);
    });
}
exports.SaveSystemDictionaries = SaveSystemDictionaries;
function SaveAzureSubscription(subscription, vcc) {
    if (subscription.azureSubscription && subscription.azureSubscription.length > 0
        && subscription.resourceGroup && subscription.resourceGroup.length > 0
        && subscription.accountName && subscription.accountName.length > 0)
        return mssql_service_1.MSSql.SaveAzureSubscription(vcc, subscription.azureSubscription, subscription.resourceGroup, subscription.accountName);
    else
        return Promise.resolve(true);
}
exports.SaveAzureSubscription = SaveAzureSubscription;
function SaveAzureSubscriptionKeys(subscriptions, vcc) {
    var promiseArray = [];
    if (subscriptions) {
        subscriptions.forEach(function (sub) {
            if (sub.subscriptionKey
                && sub.subscriptionKey.length > 0
                && sub.subscriptionName
                && sub.subscriptionName.length > 0)
                promiseArray.push(mssql_service_1.MSSql.SaveAzureSubscriptionKey(vcc, sub.subscriptionKey, sub.subscriptionRegion, sub.isDemo, sub.subscriptionName));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveAzureSubscriptionKeys = SaveAzureSubscriptionKeys;
function SaveInteractionTypes(interactions, vcc) {
    var promiseArray = [];
    if (interactions) {
        interactions.forEach(function (int) {
            promiseArray.push(mssql_service_1.MSSql.SaveInteractionType(vcc, int.id, int.description));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveInteractionTypes = SaveInteractionTypes;
function SaveReports(reports, vcc, date, user) {
    var promiseArray = [];
    if (reports) {
        reports.forEach(function (rep) {
            promiseArray.push(mssql_service_1.MSSql.SaveReport(vcc, rep.id, rep.description, rep.type, rep.procedure, rep.startDate, rep.endDate, rep.frecuency, rep.hourSpace, rep.exactTime, rep.weekday, rep.dayMonth, rep.summary, rep.filter, rep.email, date, user, date, user));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveReports = SaveReports;
