"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UtteranceMigrator = void 0;
var redis_service_1 = require("../../db/redis.service");
var log4js = require("log4js");
var mssql_service_1 = require("../../db/mssql.service");
var promises_shared_1 = require("../../shared/promises.shared");
var helper_1 = require("./helper");
var enum_1 = require("./enum");
var arrays_shared_1 = require("../../shared/arrays.shared");
var logger = log4js.getLogger('Task');
var UtteranceMigrator = (function () {
    function UtteranceMigrator() {
    }
    UtteranceMigrator.prototype.MigrateUtterances = function (vcc, data, start, end) {
        var _this = this;
        logger.info('[CognitiveEngineMigrateImplementation::MigrateUtterances] Migratint utterances for ', vcc, ' into ', data.newEngine, ' from ', start, ' to ', end);
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.LRange(enum_1.ROOT_KEY + ":" + vcc + ":Utterances:Keys", start, end - 1);
        }).then(function (keys) {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateUtterances] Found keys for utterances:', keys);
            if (keys && keys.length > 0) {
                return redis_service_1.Redis.GetCognitiveEngineMigrationModel(enum_1.ROOT_KEY + ":" + vcc + ":Utterances", arrays_shared_1.GetUniqueFromArray(keys));
            }
            return [];
        }).then(function (utterances) {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateUtterances] Found utterances:', utterances);
            if (utterances) {
                var utteranceList = [];
                for (var _i = 0, utterances_1 = utterances; _i < utterances_1.length; _i++) {
                    var u = utterances_1[_i];
                    var utterance = JSON.parse(u);
                    if (utterance) {
                        if (utterance.rawEntityLabels) {
                            utterance.entityLabels = JSON.parse(utterance.rawEntityLabels);
                        }
                        utteranceList.push(utterance);
                    }
                }
                if (utteranceList.length > 0) {
                    return _this.ProcessUtterances(vcc, data, utteranceList);
                }
            }
            return false;
        }).then(function (_) {
            if (end < data.totalUtterance) {
                return _this.MigrateUtterances(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE);
            }
            return true;
        });
    };
    UtteranceMigrator.prototype.ProcessUtterances = function (vcc, data, utterances) {
        var _this = this;
        logger.info('[CognitiveEngineMigrateImplementation::AddUtterance] Process utterance ', utterances.length, ' into ', data.newEngine, ' for ', vcc);
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, utterances_2 = utterances; _i < utterances_2.length; _i++) {
                var utterance = utterances_2[_i];
                if (utterance.status != 3) {
                    promises.push(_this.AddUtterance(vcc, data, utterance));
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::ProcessUtterances] Results for process utterances:', results);
            var totalCreated = results.filter(function (r) { return r == true; }).length;
            var totalError = results.filter(function (r) { return !r; }).length;
            return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                totalCreatedUtterance: totalCreated + data.totalCreatedUtterance,
                totalErrorUtterance: totalError + data.totalErrorUtterance
            }));
        });
    };
    UtteranceMigrator.prototype.AddUtterance = function (vcc, data, utterance) {
        logger.info('[CognitiveEngineMigrateImplementation::AddUtterance] Adding Utterance ', utterance.name, ' into ', data.newEngine);
        var AppId = null;
        var IntentId = null;
        var OriginalUtteranceId = null;
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [
                redis_service_1.Redis.GetCognitiveEngineMigrationNewId(enum_1.ROOT_KEY + ":" + vcc + ":AppIds", utterance.appId),
                redis_service_1.Redis.GetCognitiveEngineMigrationNewId(enum_1.ROOT_KEY + ":" + vcc + ":IntentIds", utterance.intentKey)
            ];
            return Promise.all(promises);
        })
            .then(function (ids) {
            logger.info('[CognitiveEngineMigrateImplementation::AddUtterance] Found ids for utterance', utterance, ':', ids);
            if (ids && ids.length == 2) {
                AppId = ids[0];
                IntentId = ids[1];
                return helper_1.GetServiceInstance(data.newEngine);
            }
            return null;
        })
            .then(function (service) {
            if (service && AppId && IntentId) {
                OriginalUtteranceId = utterance.id;
                return service.AddUtterance(data.subscriptionRegion, data.subscriptionKey, AppId, Object.assign(utterance, {
                    intent: utterance.intent
                }));
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddUtterance] Error on get new app id and new intentId from redis ', utterance);
            return null;
        }).then(function (newUtteranceId) {
            logger.info("[CognitiveEngineMigrateImplementation::AddUtterance] New utterance id:", newUtteranceId);
            if (newUtteranceId && newUtteranceId.ExampleId) {
                var promises = [];
                promises.push(Promise.resolve(newUtteranceId.ExampleId));
                promises.push(mssql_service_1.MSSql.UpdateUtteranceId(vcc, AppId, IntentId, OriginalUtteranceId, newUtteranceId.ExampleId));
                return Promise.all(promises);
            }
            logger.error("[CognitiveEngineMigrateImplementation::AddUtterance] Error on add utterance into " + data.newEngine + ":", newUtteranceId, IntentId);
            return [null];
        })
            .then(function (results) {
            var newUtteranceId = results[0];
            logger.info('[CognitiveEngineMigrateImplementation::AddIntent] Updated Utterance ID:', results);
            if (newUtteranceId) {
                var promises = [];
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(enum_1.ROOT_KEY + ":" + vcc + ":Utterances", utterance.key));
                return Promise.all(promises);
            }
            return [false];
        })
            .then(function (results) {
            return !results.includes(false);
        })
            .catch(function (err) {
            logger.error("[CognitiveEngineMigrateImplementation::AddUtterance] Unknown error on add utterance into " + data.newEngine, err);
            var promises = [];
            promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(enum_1.ROOT_KEY + ":" + vcc + ":Utterances", utterance.key, Object.assign(utterance, {
                status: 3
            })));
            promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'utterance', OriginalUtteranceId, 3, 'WITH_ERROR'));
            return Promise.all(promises).then(function () {
                return false;
            });
        });
    };
    return UtteranceMigrator;
}());
exports.UtteranceMigrator = UtteranceMigrator;
