"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAbilityQueries = exports.AbilityService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function AbilityService(app) {
    app.get('/api/ability/get_ability_detail/:abilityId', function (req, res) {
        logger.info('[AbilityService::get_ability_detail] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var id = req.params.abilityId || '';
        promises_shared_1.DummyPromise().then(function (_) {
            return mssql_service_1.MSSql.GetAbilityDetail(vcc, id);
        }).
            then(function (items) {
            res.json({ status: items ? true : false, data: items });
        }).
            catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_abilitys/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AbilityService::get_abilitys] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilitys(instance, search_criteria, search_expression, disabled);
        }).then(function (abilitys) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: abilitys });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/create_ability', function (req, res, next) {
        logger.info('[AbilityService::create_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var wasCreated = false;
        var newAbility = (req.body || {});
        newAbility.createdDate = nowUtc;
        newAbility.createdByUserId = userId;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        newAbility.title = newAbility.title.trim();
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetAbilityTitle(vcc, newAbility.title);
        })
            .then(function (result) {
            if (result)
                throw 'ABILITY_SERVER_ERROR_EXISTS';
            newAbility.id = utilities_1.guid();
            return SaveAbility(vcc, newAbility);
        })
            .then(function (result) {
            var _a;
            if (!result)
                throw 'ABILITY_SERVER_ERROR_ON_SAVE';
            wasCreated = true;
            if ((_a = newAbility === null || newAbility === void 0 ? void 0 : newAbility.queries) === null || _a === void 0 ? void 0 : _a.length)
                return SaveAbilityQueries(vcc, newAbility, userId);
            else
                return Promise.resolve(true);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_ABILITY_QUERY_SAVE';
            return SaveScore(vcc, newAbility, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(newAbility.id);
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'Abilidad save failed'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            if (wasCreated)
                mssql_service_1.MSSql.DeleteAbility(vcc, newAbility.id);
            logger.error('[AbilityService::create_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_ability/:ability_id', function (req, res, next) {
        logger.info('[AbilityService::get_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var ability = {};
        var scores = [];
        var abilityId = req.params.ability_id;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!abilityId)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetAbility(vcc, abilityId);
        })
            .then(function (result) {
            if (!abilityId)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            ability = result || ability;
            return GetAbilityQueries(vcc, abilityId, true);
        })
            .then(function (result) {
            ability.queries = result;
            return mssql_service_1.MSSql.GetScores(vcc, abilityId);
        })
            .then(function (resultScores) {
            if (res.headersSent)
                return;
            scores = resultScores || scores;
            ability.scores = scores;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: ability });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/save_ability', function (req, res, next) {
        logger.info('[AbilityService::save_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newAbility = (req.body || {});
        newAbility.createdDate = null;
        newAbility.createdByUserId = null;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilityTitle(vcc, newAbility.title);
        }).then(function (result) {
            var abilityDesc = {};
            abilityDesc = result || abilityDesc;
            if (result && abilityDesc.id !== newAbility.id)
                throw 'ABILITY_SERVER_ERROR_EXISTS';
            return mssql_service_1.MSSql.SaveAbility(vcc, newAbility.id, newAbility.title, newAbility.detail, newAbility.type, newAbility.calificationType, newAbility.binary, newAbility.notApply, newAbility.distribute, newAbility.acceptComments, newAbility.createdDate, newAbility.createdByUserId, newAbility.lastModifiedDate, newAbility.lastModifiedByUserId, newAbility.enabled, newAbility.analysisType, newAbility.language, newAbility.isNegative);
        })
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteAbilityQueries(vcc, newAbility.id);
        })
            .then(function (result) {
            if (newAbility.queries && newAbility.queries.length > 0) {
                return SaveAbilityQueries(vcc, newAbility, userId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (resultIdAbility) {
            return mssql_service_1.MSSql.DeleteScore(vcc, newAbility.id);
        }).then(function (resultDeleteScore) {
            return SaveScore(vcc, newAbility, userId);
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_ON_SAVE'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::save_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_ability', function (req, res, next) {
        logger.info('[AbilityService::delete_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var ability = (req.body || {});
        var abilityId = ability.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!ability)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteScore(vcc, abilityId);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteAbility(vcc, abilityId);
        }).then(function (result) {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_CANNOT_DELETE';
            return Promise.resolve(true);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_CANNOT_DELETE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::delete_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_abilitys', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var abilitys = req.body.abilitys || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!abilitys)
                throw 'ABILITY_SERVER_ERROR_MISING_VCC';
            var promiseArray = [];
            abilitys.forEach(function (ability) {
                promiseArray.push(mssql_service_1.MSSql.DeleteScore(vcc, ability.id));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!abilitys)
                throw 'ABILITY_SERVER_ERROR_MISING_VCC';
            var promiseArray = [];
            abilitys.forEach(function (ability) {
                promiseArray.push(mssql_service_1.MSSql.DeleteAbility(vcc, ability.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteVccs) {
            var result;
            if (resultDeleteVccs.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Vccs eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteVccs) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteVccs, description: (resultDeleteVccs ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_CANNOT_DELETE'), data: resultDeleteVccs });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AbilityService = AbilityService;
function SaveAbility(vcc, ability) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAbility(vcc, ability.id, ability.title, ability.detail, ability.type, ability.calificationType, ability.binary, ability.notApply, ability.distribute, ability.acceptComments, ability.createdDate, ability.createdByUserId, ability.lastModifiedDate, ability.lastModifiedByUserId, ability.enabled, ability.analysisType, ability.language, ability.isNegative));
    return Promise.all(promiseArray)
        .then(function (results) {
        var falseResults = results.filter(function (res) { return res[0].result !== true && res[0].result !== 1; });
        if (falseResults.length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveAbilityQueries(vcc, ability, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        if (ability && ability.queries && ability.queries.length > 0) {
            ability.queries.map(function (aq) {
                aq.id = utilities_1.guid();
                aq.abilityId = ability.id;
                promiseArray.push(SaveAbilityQuery(vcc, aq, userId));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var saveResult = false;
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            saveResult = false;
        }
        else {
            saveResult = true;
        }
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::SaveAbilityQueries] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function SaveAbilityQuery(vcc, abilityQuery, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.SaveAbilityQuery(vcc, abilityQuery.id, abilityQuery.abilityId, abilityQuery.queryId, abilityQuery.queryContent, abilityQuery.acceptancePercentage, abilityQuery.order, userId, userId);
    })
        .then(function (result) {
        var promiseArray = [];
        if (abilityQuery.entities && abilityQuery.entities.length > 0) {
            abilityQuery.entities.map(function (e) {
                e.abilityQueryId = abilityQuery.id;
                promiseArray.push(SaveAbilityQueryEntity(vcc, e, userId));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var saveResult = false;
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            saveResult = false;
        }
        else {
            saveResult = true;
        }
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::SaveAbilityQuery] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function SaveAbilityQueryEntity(vcc, abilityQueryEntity, userId) {
    return mssql_service_1.MSSql.SaveAbilityQueryEntity(vcc, abilityQueryEntity.abilityQueryId, abilityQueryEntity.entityId, abilityQueryEntity.entityName, abilityQueryEntity.acceptancePercentage, abilityQueryEntity.order, userId, userId);
}
function GetAbilityQueries(vcc, abilityId, getEntities, evaluationPlanId) {
    if (getEntities === void 0) { getEntities = false; }
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    var abilityQueries = [];
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return mssql_service_1.MSSql.GetAbilityQueries(vcc, abilityId, evaluationPlanId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_ABILITY_QUERIES_NOT_FOUND';
        if (getEntities) {
            return GetAbilityQueryEntities(vcc, result, evaluationPlanId);
        }
        else {
            result.map(function (r) { return r.entities = []; });
            return Promise.resolve(result);
        }
    })
        .then(function (result) {
        abilityQueries = result;
        return Promise.resolve(abilityQueries);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[AbilityService::GetAbilityQueries] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAbilityQueries = GetAbilityQueries;
function GetAbilityQueryEntities(vcc, abilityQueries, evaluationPlanId) {
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        var promiseArray = [];
        abilityQueries.map(function (aq) {
            promiseArray.push(mssql_service_1.MSSql.GetAbilityQueryEntities(vcc, aq.id, evaluationPlanId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        result.map(function (r, i) {
            abilityQueries[i].entities = r;
        });
        return Promise.resolve(abilityQueries);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::GetAbilityQueryEntities] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
function SaveScore(vcc, ability, userId) {
    var _a;
    var promiseArray = [];
    var hasTipifications = false;
    if ((_a = ability.scores) === null || _a === void 0 ? void 0 : _a.length) {
        ability.scores.forEach(function (score) {
            var _a;
            promiseArray.push(mssql_service_1.MSSql.SaveScore(vcc, score.id, ability.id, score.description, score.value));
            if ((_a = score === null || score === void 0 ? void 0 : score.tipifications) === null || _a === void 0 ? void 0 : _a.length)
                hasTipifications = true;
        });
    }
    return Promise.all(promiseArray)
        .then(function (_) {
        return mssql_service_1.MSSql.GetScores(vcc, ability.id);
    })
        .then(function (result) {
        promiseArray = [];
        if (hasTipifications) {
            ability.scores.filter(function (score) { return score.tipifications; }).forEach(function (score) {
                var _a;
                score.tipifications = score.tipifications.filter(function (t) { return t; });
                var savedScore = result.find(function (s) { return s.description === score.description && s.value === score.value; });
                score.id = savedScore.id;
                var classifiedTipifications = arrays_shared_1.GetCompareItemsFromObjectArray(savedScore.tipifications, score.tipifications, 'id');
                score.tipifications.forEach(function (t) {
                    var _a;
                    var savedTipification = savedScore.tipifications.find(function (st) { return st.id === t.id; });
                    var classifiedTipificationItems = arrays_shared_1.GetCompareItemsFromObjectArray((savedTipification === null || savedTipification === void 0 ? void 0 : savedTipification.items) ? savedTipification.items : [], t.items, 'id');
                    promiseArray.push(mssql_service_1.MSSql.SaveScoreTipification(vcc, score.id, t.id, t.description, t.descriptionDetail, t.order, userId));
                    promiseArray.push(mssql_service_1.MSSql.SaveScoreTipificationQuery(vcc, score.id, t.id, t.tipificationQuery.queryId, t.tipificationQuery.queryContent, t.tipificationQuery.acceptancePercentage, userId));
                    t.items.forEach(function (i) {
                        promiseArray.push(mssql_service_1.MSSql.SaveScoreTipificationItem(vcc, score.id, t.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
                        if (t.isNew) {
                            promiseArray.push(mssql_service_1.MSSql.SaveTipificationItem(vcc, t.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
                            promiseArray.push(mssql_service_1.MSSql.SaveTipificationQuery(vcc, t.id, t.tipificationQuery.queryId, t.tipificationQuery.queryContent, t.tipificationQuery.acceptancePercentage, userId));
                        }
                        ;
                    });
                    if ((_a = classifiedTipificationItems === null || classifiedTipificationItems === void 0 ? void 0 : classifiedTipificationItems.removed) === null || _a === void 0 ? void 0 : _a.length)
                        classifiedTipificationItems.removed.forEach(function (i) { return promiseArray.push(mssql_service_1.MSSql.DeleteScoreTipificationItem(vcc, score.id, i.id)); });
                });
                if ((_a = classifiedTipifications === null || classifiedTipifications === void 0 ? void 0 : classifiedTipifications.removed) === null || _a === void 0 ? void 0 : _a.length)
                    classifiedTipifications.removed.forEach(function (i) { return promiseArray.push(mssql_service_1.MSSql.DeleteScoreTipification(vcc, score.id, i.id)); });
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length)
            return Promise.resolve(false);
        else
            return Promise.resolve(true);
    });
}
